Module tools

Module tools 

Utility routines.

Functions

communicate_back
Communicate the first element of each local array back to the previous rank.
displacements
Compute displacements from a vector of counts.
gather_to_all
Gather array to all processes
gather_to_root
Array to root Gather distributed array to the root rank.
generate_random_keys
Generate random keys for testing.
generate_random_points
Generate random points for testing.
global_inclusive_cumsum
Perform a global inclusive cumulative sum operation.
global_max
Get the maximum value across all ranks
global_min
Get the minimum value across all ranks
global_size
Get global size of a distributed array.
is_sorted_array
Check if an array is sorted.
redistribute
Redistribute an array via an all_to_all_varcount operation.
redistribute_by_bins
Redistribute locally sorted keys with respect to bins.
seeded_rng
Get a seeded rng
sort_to_bins
Distribute a sorted sequence into bins.