Utility routines.
Functions
- communicate_
back - Communicate the first element of each local array back to the previous rank.
- displacements
- Compute displacements from a vector of counts.
- gather_
to_ all - Gather array to all processes
- gather_
to_ root - Array to root Gather distributed array to the root rank.
- generate_
random_ keys - Generate random keys for testing.
- generate_
random_ points - Generate random points for testing.
- global_
inclusive_ cumsum - Perform a global inclusive cumulative sum operation.
- global_
max - Get the maximum value across all ranks
- global_
min - Get the minimum value across all ranks
- global_
size - Get global size of a distributed array.
- is_
sorted_ array - Check if an array is sorted.
- redistribute
- Redistribute an array via an all_to_all_varcount operation.
- redistribute_
by_ bins - Redistribute locally sorted keys with respect to bins.
- seeded_
rng - Get a seeded rng
- sort_
to_ bins - Distribute a sorted sequence into bins.